/*
 * CodePress regular expressions for PHP syntax highlighting
 */

// PHP
Language.syntax = [
	{ input : /(&lt;[^!\?]*?&gt;)/g, output : '<b>$1</b>' }, // all tags
	{ input : /(&lt;style.*?&gt;)(.*?)(&lt;\/style&gt;)/g, output : '<em>$1</em><em>$2</em><em>$3</em>' }, // style tags
	{ input : /(&lt;script.*?&gt;)(.*?)(&lt;\/script&gt;)/g, output : '<ins>$1</ins><ins>$2</ins><ins>$3</ins>' }, // script tags
	{ input : /\"(.*?)(\"|<br>|<\/P>)/g, output : '<s>"$1$2</s>' }, // strings double quote
	{ input : /\'(.*?)(\'|<br>|<\/P>)/g, output : '<s>\'$1$2</s>'}, // strings single quote
	{ input : /(&lt;\?)/g, output : '<strong>$1' }, // <?.*
	{ input : /(\?&gt;)/g, output : '$1</strong>' }, // .*?>
	{ input : /(&lt;\?php|&lt;\?=|&lt;\?|\?&gt;)/g, output : '<cite>$1</cite>' }, // php tags
	{ input : /(\$[\w\.]*)/g, output : '<a>$1</a>' }, // vars
	{ input : /\b(false|true|and|or|xor|__FILE__|exception|__LINE__|array|as|break|case|class|const|continue|declare|default|die|do|echo|else|elseif|empty|enddeclare|endfor|endforeach|endif|endswitch|endwhile|eval|exit|extends|for|foreach|function|global|if|include|include_once|isset|list|new|print|require|require_once|return|static|switch|unset|use|while|__FUNCTION__|__CLASS__|__METHOD__|final|php_user_filter|interface|implements|extends|public|private|protected|abstract|clone|try|catch|throw|this)\b/g, output : '<u>$1</u>' }, // reserved words
	{ input : /\b(array_keysSearch|array_makeFirst|asc|ASSI_addClient|ASSI_addJobs|ASSI_addUbuntuRoot|ASSI_prepareClient|ASSI_showClientAddDialog|AUTOTEST_calcScancodes|AUTOTEST_createLDAPUserAndGroup|AUTOTEST_executePHPFunction|AUTOTEST_getKey|AUTOTEST_getSeleniumSafeString|AUTOTEST_getSeleniumUnsafeString|AUTOTEST_isDebug|AUTOTEST_keyAndRelease|AUTOTEST_replaceConstantsInString|AUTOTEST_SELENIUM_clickButtonID|AUTOTEST_SELENIUM_clickButtonName|AUTOTEST_SELENIUM_close|AUTOTEST_SELENIUM_deselectFromID|AUTOTEST_SELENIUM_deselectFromName|AUTOTEST_SELENIUM_exec|AUTOTEST_SELENIUM_getsource|AUTOTEST_SELENIUM_open|AUTOTEST_SELENIUM_quit|AUTOTEST_SELENIUM_selectFromID|AUTOTEST_SELENIUM_selectFromName|AUTOTEST_SELENIUM_selectRadioID|AUTOTEST_SELENIUM_selectRadioName|AUTOTEST_SELENIUM_setCheckID|AUTOTEST_SELENIUM_setCheckName|AUTOTEST_SELENIUM_typeIntoID|AUTOTEST_SELENIUM_typeIntoName|AUTOTEST_SEL_getURLByMatch|AUTOTEST_sshTunnelOverServer|AUTOTEST_sshVMServer|AUTOTEST_VM_create|AUTOTEST_VM_delete|AUTOTEST_VM_enableCapture|AUTOTEST_VM_export_m23ServerISO_as_OVA|AUTOTEST_VM_getStatus|AUTOTEST_VM_hostSanityCheck|AUTOTEST_VM_insertBootISO|AUTOTEST_VM_isRunning|AUTOTEST_VM_keyboardWrite|AUTOTEST_VM_ocrScreen|AUTOTEST_VM_rebootFromHD|AUTOTEST_VM_restoreSnapshot|AUTOTEST_VM_screenPixelDiff|AUTOTEST_VM_start|AUTOTEST_VM_stop|BACKUP_addAdmin|BACKUP_delAdmin|BACKUP_getAdmins|BACKUP_getBackupDirs|BACKUP_saveBackupDirs|BACKUP_showClientSettings|base64|base64_encode|base64_write|BURN_blank|BURN_burn|BURN_checkISO|BURN_getDevice|BURN_getISOSize|BURN_getStatus|BURN_listBurners|BURN_showLog|CALC_add_dec|CALC_add_equals|CALC_add_number|CALC_add_sign|CALC_showCalculator|CAPTURE_captureAll|CAPTURE_captureImg|CAPTURE_deActivate|CAPTURE_deleteById|CAPTURE_getKeys|CAPTURE_isActive|CAPTURE_load|CAPTURE_showEntries|CAPTURE_showMarker|CAPTURE_showMessageBox|CAPTURE_showTableWith|CAPTURE_toggle|CHECK_db2text|CHECK_deviceName|CHECK_diskFree|CHECK_float|CHECK_FW|CHECK_int|CHECK_ip|CHECK_letFWDie|CHECK_mointPoint|CHECK_safeFilename|CHECK_str|CHECK_strAlpha|CHECK_strAlphaNum|CHECK_text2db|CLCFG_activateBOOT_DEGRADED|CLCFG_activateDMA|CLCFG_addDistributionSpecificOptions|CLCFG_addGrubPassword|CLCFG_addLiloPassword|CLCFG_addPAMtoDM|CLCFG_addUser|CLCFG_aptConf|CLCFG_aptGet|CLCFG_blacklistPackageInstallation|CLCFG_changeUser|CLCFG_configUpstartForChroot|CLCFG_configUpstartForNormalUsage|CLCFG_copyClientPackageStatus|CLCFG_copyMBRToAllDevices|CLCFG_copySSLCert|CLCFG_createBootDeviceNode|CLCFG_createScreenRC|CLCFG_debootstrap|CLCFG_dialogAllBox|CLCFG_dialogGaugeProcPos|CLCFG_dialogInfoBox|CLCFG_dialogMsgBox|CLCFG_disableAptSystemdDaily|CLCFG_disableAvahiDaemon|CLCFG_disableNFSHome|CLCFG_disablePlymouth|CLCFG_disableSudoRootLogin|CLCFG_downgradeExt|CLCFG_efi|CLCFG_enableLDAPDebian|CLCFG_enableLDAPUbuntu|CLCFG_enableNFSHome|CLCFG_executeAfterChroot|CLCFG_executeNextWorkEveryMinute|CLCFG_fetchm23BasicTools|CLCFG_genFakeFstab|CLCFG_genFstab|CLCFG_getDebianReleasesGeneric|CLCFG_getHSReleasesGeneric|CLCFG_getMbrPart|CLCFG_getRootDeviceFS|CLCFG_hideKernelWarnings|CLCFG_hostname|CLCFG_hwdetect|CLCFG_importLocalPoolKey|CLCFG_installApplicationLanguagePackages|CLCFG_installBasePackages|CLCFG_installDesktopLanguagePackage|CLCFG_installFirmware|CLCFG_installLightDM|CLCFG_installMintDM|CLCFG_installSDDM|CLCFG_installXDM|CLCFG_interfaces|CLCFG_language|CLCFG_lilo2Grub|CLCFG_listDebianReleases|CLCFG_listDebianReleasesGeneric|CLCFG_listHSReleases|CLCFG_listHSReleasesGeneric|CLCFG_listUbuntuReleases|CLCFG_makeDev|CLCFG_monoRemove|CLCFG_mountRootDir|CLCFG_patchNsswitchForLDAP|CLCFG_resolvConf|CLCFG_setAuthorized_keys|CLCFG_setDebconf|CLCFG_setDebConfDirect|CLCFG_setDebConfDM|CLCFG_setRootPassword|CLCFG_setSystime|CLCFG_setTimeZone|CLCFG_showDistributionSpecificOptions|CLCFG_showDistributionSpecificOptionsUbuntu|CLCFG_sourceslist|CLCFG_sourceslistCreateConfigFiles|CLCFG_updateDebootstrapScripts|CLCFG_updateDebootstrapScriptsDebian|CLCFG_updateDebootstrapScriptsUbuntu|CLCFG_writeCrontabm23fetchjobEvery5Minutes|CLCFG_writeHosts|CLCFG_writeM23fetchjob|CLDI_load|CLDI_save|CLIENT_acceptChanges|CLIENT_addChangeElement|CLIENT_addClient|CLIENT_addClientlogsFailure|CLIENT_addClientlogsOk|CLIENT_addClientlogsUnknown|CLIENT_backToRed|CLIENT_changeClient|CLIENT_convertMac|CLIENT_copyDebconfDB|CLIENT_deleteClient|CLIENT_desasterRecovery|CLIENT_DETAILS_addIcon|CLIENT_DETAILS_addIcon2|CLIENT_DETAILS_beginCategory|CLIENT_DETAILS_endCategory|CLIENT_executeOnClientOrIP|CLIENT_exists|CLIENT_extraWebAction|CLIENT_filterLinesFromLiveScreenRecording|CLIENT_generateHTMLDedicatedAndReachableStatus|CLIENT_generateHTMLStatusBar|CLIENT_getActiveNetDevices|CLIENT_getAllAskingOptions|CLIENT_getAllAsRes|CLIENT_getAllClientNames|CLIENT_getAllOptions|CLIENT_getAskingParams|CLIENT_getBootType|CLIENT_getBroadcast|CLIENT_getClientAmount|CLIENT_getClientID|CLIENT_getClientName|CLIENT_getClientStatus|CLIENT_getCurrentFreeSpaceInDir|CLIENT_getCurrentMemoryUsage|CLIENT_getDebconfDB|CLIENT_getDebconfDBValue|CLIENT_getDebugimage|CLIENT_getDistribution|CLIENT_getId|CLIENT_getIPbyName|CLIENT_getMACbyName|CLIENT_getNamebyIP|CLIENT_getNames|CLIENT_getNamesWithPackages|CLIENT_getNetmaskBits|CLIENT_getNetmaskFromBitAmount|CLIENT_getNextFreeIp|CLIENT_getOption|CLIENT_getOverviewSearchLine|CLIENT_getParams|CLIENT_getPossibleActions|CLIENT_getProperty|CLIENT_getSetOption|CLIENT_getSSHKeyorPasswordOptions|CLIENT_getStatusimage|CLIENT_getSubnet|CLIENT_getToDetailsURL|CLIENT_hiddenForm2options|CLIENT_HTMLBackToDetails|CLIENT_insertIntoClientlogs|CLIENT_IPexists|CLIENT_isAskingInDebugMode|CLIENT_isAssimilated|CLIENT_isBasesystemInstalledFromImage|CLIENT_isDedicatedAndReachable|CLIENT_isInDebugMode|CLIENT_isInRescueMode|CLIENT_isrunning|CLIENT_listCriticalClients|CLIENT_listPackages|CLIENT_liveLogJobName|CLIENT_MACexists|CLIENT_options2HiddenForm|CLIENT_plinkFetchJob|CLIENT_query|CLIENT_recalculateStatusBar|CLIENT_recoverClientCombineJobs|CLIENT_removeServerCache|CLIENT_reset|CLIENT_resetAndInstall|CLIENT_resetStatusBar|CLIENT_runDebconf|CLIENT_sendM23FetchJobFinishedMessage|CLIENT_sendNotifySendMessage|CLIENT_sendPlymouthMessage|CLIENT_setAllOptions|CLIENT_setAllParams|CLIENT_setDebconfDB|CLIENT_setLastmodify|CLIENT_showAddDialog|CLIENT_showDebugSelection|CLIENT_showDelDialog|CLIENT_showDirectConnectionHelp|CLIENT_showGeneralInfo|CLIENT_showHardwareInfo|CLIENT_showJobs|CLIENT_showLastLogError|CLIENT_showLog|CLIENT_showOverviewSearchDialog|CLIENT_showStatusSelection|CLIENT_sshFetchJob|CLIENT_startInstall|CLIENT_startLiveScreenRecording|CLIENT_stopLiveScreenRecording|CLIENT_toggleDebugMode|CLIENT_touchLiveLogFile|CLIENT_touchLogFile|CLIENT_wol|CRON_addJobBasic|CRON_addJobDayly|CRON_addJobHourly|CRON_addJobMinutely|CRON_addJobWeekly|CRON_checkHour|CRON_checkMinute|CRON_cronEntryDeletionDialog|CRON_cronManagementDialog|CRON_genCronEntry|CRON_getDayOfWeekArray|CRON_getEntriesByIdentifier|CRON_getNextIdentifierNr|CRON_getTimeBaseArray|CRON_isEntryPresent|CRON_reloadConfig|CRON_rmJob|CRON_translateEveryIntervallValue|CSYSTEMPROXY_addCurlProxySettings|CSYSTEMPROXY_getAptGetProxyParamter|CSYSTEMPROXY_getEnvironmentVariables|CSYSTEMPROXY_getProxySettingsFromAPT|CSYSTEMPROXY_getUserPasswordString|DB_changeAllCollations|DB_genPassword|DB_getArrayAssoc|DB_getConnection|DB_getLikeableColumns|DB_getSuperUserName|DB_getSuperUserPassword|DB_getTableColumns|DB_isConnectionValid|DB_query|DB_queryNoDie|DB_setConnection|DEBIAN_desktopInstall|DEBUG_returnNotQ|DEBUG_returnV|DHCP_activateBoot|DHCP_addClient|DHCP_addDynamicRange|DHCP_addLineToDHCPDConf|DHCP_addSubnetDefinition|DHCP_bootTypeToNewFormat|DHCP_calcPXEIP|DHCP_delDynamicRange|DHCP_delSubnetDefinition|DHCP_exportDHCPSettingsForExternalDHCPServer|DHCP_getDynamicRanges|DHCP_getNetmaskOfDynamicRanges|DHCP_isNetworkBootingActive|DHCP_lineNumberAffterLastClient|DHCP_removePXEcfg|DHCP_restartDHCPserver|DHCP_rmClient|DHCP_runScript|DHCP_setBootimage|DHCP_writePXEcfg|DIALOG_importDirFiles|DISTR_afterChrootInstall|DISTR_baseInstall|DISTR_commaSeperatedSelections|DISTR_DistributionsSelections|DISTR_getDebianUserGroups|DISTR_getDescriptionValues|DISTR_getDesktopDescription|DISTR_getDesktopsCBList|DISTR_geti18nValue|DISTR_getSelectedDesktopsArr|DISTR_getSelectedDesktopsStr|DISTR_getUbuntuUserGroups|DISTR_listCommaSeperated|DISTR_listDistributions|DISTR_releaseVersionTranslator|DISTR_startInstall|DMI_getAllTextBox|DMI_getBoard|DMI_getCPU|DMI_getMemory|DMI_getParam|DMI_getSlot|EDIT_addIfNotExists|EDIT_appendToFile|EDIT_calc|EDIT_commentout|EDIT_commentoutAll|EDIT_commentoutInsert|EDIT_countMatches|EDIT_deleteLines|EDIT_deleteLinesAmount|EDIT_deleteMatching|EDIT_genClientm23Random|EDIT_insertAfterLineNumber|EDIT_insertAtLineNumber|EDIT_insertLineNumber|EDIT_prepareStr|EDIT_replace|EDIT_replaceLineElseAdd|EDIT_restorePerms|EDIT_savePerms|EDIT_searchLastLineNumber|EDIT_searchLineNumber|EDIT_searchNextLineNumber|EDIT_sedSearchAddInsert|EDIT_setOption|EDIT_uncomment|EDIT_writeToFile|FDISK_addDrivePartitionToRaid|FDISK_addFstab|FDISK_addJob|FDISK_addPart|FDISK_addRaidBeforeFormat|FDISK_addRaidJobs|FDISK_adjustFdiskParams|FDISK_adjustFstabParam|FDISK_AFPgetPartSizes|FDISK_AFPlinearScale|FDISK_AFPselectDrive|FDISK_autoPart|FDISK_bootflagJob|FDISK_buildRaidDialog|FDISK_canPartTypeBeCreated|FDISK_checkFreeSpace|FDISK_colorFS|FDISK_correctLogical|FDISK_countPartitions|FDISK_defineDrive|FDISK_definePartitionSelection|FDISK_deleteDriveFromParam|FDISK_deletePartitionFromParam|FDISK_delFstab|FDISK_delPart|FDISK_dev2LDevLPart|FDISK_devNrExists|FDISK_fdiskSessionClient|FDISK_fdiskSessionExtendedPartStep|FDISK_fdiskSessionFreeSpaces|FDISK_fdiskSessionFstab|FDISK_fdiskSessionHelpPage|FDISK_fdiskSessionInstallDrive|FDISK_fdiskSessionInstPart|FDISK_fdiskSessionPage|FDISK_fdiskSessionParam|FDISK_fdiskSessionPartJobs|FDISK_fdiskSessionPartMethod|FDISK_fdiskSessionReset|FDISK_fdiskSessionSetter|FDISK_fdiskSessionSwapPart|FDISK_fdiskSessionTitle|FDISK_fdiskSessionvDevInstall|FDISK_finalChecksAndRealPartitionAndFormatStart|FDISK_findDevNrPosition|FDISK_findFstabMountPointByDev|FDISK_formatExisting|FDISK_formatInstallAndSwappart|FDISK_formatJob|FDISK_formatPart|FDISK_fstabAddDialog|FDISK_fstabAddDialog2|FDISK_genManualFstab|FDISK_genPartedCommands|FDISK_getAfterPartition|FDISK_getAllDrives|FDISK_getBeforeFristPartition|FDISK_getBelongingRaidDev|FDISK_getBiggestValueOf|FDISK_getDiskType|FDISK_getDriveAndNr|FDISK_getDriveInfoIcon|FDISK_getDriveInfoString|FDISK_getDrivePartitionSize|FDISK_getDrivesAndPartitions|FDISK_getFirstDrive|FDISK_getFreeSpaces|FDISK_getFstabArray|FDISK_getNextFdiskFormatJobNr|FDISK_getPartInfoIcon|FDISK_getPartInfoString|FDISK_getPartitionByType|FDISK_getPartitionPercent|FDISK_getPartitions|FDISK_getPartitionsFromParam|FDISK_getSupportedFS|FDISK_getUnusedMDs|FDISK_getvPart|FDISK_installExistingDialog|FDISK_installFilesystems|FDISK_listDrivesAndPartitions|FDISK_listDrivesAndPartitions2|FDISK_listFstab|FDISK_listInstPartSelector|FDISK_listPartitions|FDISK_listPartJobs|FDISK_listPartTable|FDISK_listRaidTable|FDISK_listSupportedFS|FDISK_mdToEndOfArray|FDISK_nextLogicalDevNr|FDISK_nextPrimaryDevNr|FDISK_partCreationSelect|FDISK_printAllBars|FDISK_printAllBars2|FDISK_printBars|FDISK_printColorDefinitions|FDISK_rereadPartTable|FDISK_rmJob|FDISK_selectDrives|FDISK_showAllPartTables|FDISK_showCombinedFdiskGUIDialog|FDISK_showDiskDefine|FDISK_showFdiskCombinedGUIFunctions|FDISK_showFdiskGUIAutoMethod|FDISK_showFdiskGUIDialog|FDISK_showFdiskGUIDriveInfoDialog|FDISK_showFdiskGUIExistingMethod|FDISK_showFdiskGUIExtendedMethod|FDISK_showFdiskGUIExtendedStepAddNewPartition|FDISK_showFdiskGUIExtendedStepDeletePartitions|FDISK_showFdiskGUIExtendedStepFormatPartitions|FDISK_showFdiskGUIlistPartJobs|FDISK_showFdiskGUISelectPartitionMethod|FDISK_showFdiskGUITitle|FDISK_swapFilesystems|FDISK_virtualAddDrive|FDISK_virtualAddPartition|FDISK_virtualDeleteDrive|FDISK_virtualDeletePartition|GNOME3_install|GNOME_install|GNOME_installLoginManager|GNOME_prepare|GRP_add|GRP_addClientToGroup|GRP_countClients|GRP_del|GRP_delClientFromGroup|GRP_desasterRecovery|GRP_doClientMoreGroups|GRP_exists|GRP_getAllPackages|GRP_getDescrGroup|GRP_getDistrsAndSourcesLists|GRP_getIdByName|GRP_getNameById|GRP_groupSelection|GRP_HTMLBackToDetails|GRP_HTMLBackToOverview|GRP_isClientInGroup|GRP_listAllClientsInGroup|GRP_listAllClientsInGroups|GRP_listClientGroups|GRP_listGroups|GRP_listGroupsAndCount|GRP_moveClientToGroup|GRP_ren|GRP_setDescrGroup|GRP_showChangeDescriptionDialog|GRP_showClientGroups|GRP_showDelDialog|GRP_showGeneralInfo|GRP_showGroupsAndCount|GRP_showRenDialog|GRP_showSelDistrSources|HELPER_array2AssociativeArray|HELPER_array2m23Array|HELPER_arrayInsertAfterKeynumber|HELPER_arrayInsertBeforeKeynumber|HELPER_arrayReOrderKeynumbers|HELPER_calcMBSize|HELPER_compareLengthAbc|HELPER_createSelfSignedCAAndServerCertificate|HELPER_debugBacktraceToFile|HELPER_filesize|HELPER_filterOutUnwantedSSHErrors|HELPER_generateSalt|HELPER_getApacheGroup|HELPER_getApacheUser|HELPER_getBootLoaders|HELPER_getClientNameAndID|HELPER_getContentFromURL|HELPER_getFdiskMountPoints|HELPER_getFileContents|HELPER_getNewLogLines|HELPER_getRemoteFileContents|HELPER_getTimeZones|HELPER_grep|HELPER_grepCount|HELPER_grepNot|HELPER_grubMd5Crypt|HELPER_hostname2IP|HELPER_importAllIntoPOST|HELPER_indentLines|HELPER_isExecutedInCLI|HELPER_isExecutedOnUCS|HELPER_isUpper|HELPER_listFilesInDir|HELPER_logToFile|HELPER_m23Array2Array|HELPER_maxPhpUploadSize|HELPER_md5x5|HELPER_netmaskAmountOfSetBits|HELPER_netmaskCalculator|HELPER_networkCalculator|HELPER_passGenerator|HELPER_putFileContents|HELPER_randomMAC|HELPER_randomUsername|HELPER_resetNewLogLines|HELPER_rmRecursive|HELPER_showBAfH|HELPER_showFileContents|HELPER_showScriptFooter|HELPER_showScriptHeader|HELPER_sortByLength|HELPER_splitDayHourMinuteString|HELPER_str_equal_UTF8ISO|HELPER_trimValue|HELPER_ucrc32|HELPER_URIencode|HELPER_xargsRecursive|HELP_getHelp|HELP_getHelpString|HELP_showHelp|HELP_showHelpTex|HS_ASSI_getClientSettingsCommand|HS_ASSI_prepareClient|HS_ASSI_statusFileCommand|HS_fetchAndExtractOSImage|HS_fetchm23HSAdminAndm23hwscanner|HS_fetchm23HSAdminAndm23hwscannerByOS|HS_getClientSettingsCommand|HS_getm23HSAdminPath|HS_getPackageCacheName|HS_hookBeginAfterChroot|HS_hookEndAfterChroot|HS_netConfig|HS_netDisableNTP|HS_netEnableLDAP|HS_netEnableNFSHome|HS_netEnableNTP|HS_netEnableSSHdAndImportKey|HS_netSetm23SSLCertificate|HS_netWriteHosts|HS_normalUpdate|HS_pkgDeinstall|HS_pkgDeinstallPreview|HS_pkgFullUpdate|HS_pkgInstall|HS_pkgInstallBasePackages|HS_pkgInstalledList|HS_pkgInstallGnome|HS_pkgInstallKDE|HS_pkgInstallLXDE|HS_pkgInstallPreview|HS_pkgInstallX|HS_pkgInstallXFce|HS_pkgSearch|HS_pkgUpdateCache|HS_pkgUpdateCacheOnServer|HS_runClientPackageConfDB|HS_sendCommandExecutionStatus|HS_setPackageProxy|HS_setSourcesList|HS_statusFileCommand|HS_sysAddFstabEntries|HS_sysAddUser|HS_sysChangeUser|HS_sysConfigurePrinter|HS_sysHWsetup|HS_sysInstallKernel|HS_sysInstallPrinter|HS_sysMakeBootable|HS_sysSetLanguage|HS_sysSetm23ClientID|HS_sysSetRootPW|HS_sysSetTimeZone|HS_sysWriteCrontabm23fetchjobEvery5Minutes|HS_sysWriteM23fetchjob|HS_wrapper|HS_wrapperReturn|HTML_checkBox|HTML_checkboxChangerButtons|HTML_checkBoxCheckAll|HTML_esel|HTML_getElementValue|HTML_getOriginalUploadFilename|HTML_getQuestionnaireURL|HTML_getStatusBarID|HTML_getValidSelected|HTML_hiddenVar|HTML_imgSwitch|HTML_incStatusBarPercentByName|HTML_input|HTML_jQueryMenu|HTML_jQueryMenuEnd|HTML_jQueryMenuHeader|HTML_jQueryReStoreYWindowPosition|HTML_jsCheckboxChanger|HTML_JSMenuCloseAllEntries|HTML_JSMenuOpener|HTML_listSelection|HTML_liveLogArea|HTML_liveSpan|HTML_logArea|HTML_manipulateOutputBuffer|HTML_multiCheckBox|HTML_multiCheckBoxShow|HTML_newStatusBar|HTML_questionnaire|HTML_selection|HTML_setPage|HTML_setStatusBarPercentPointByName|HTML_setStatusBarStatus|HTML_setStatusBarStatusByID|HTML_setStatusBarStatusByName|HTML_showFormEnd|HTML_showFormHeader|HTML_showPagePrintButton|HTML_showSmallTitle|HTML_showStatusBar|HTML_showStatusBarHTML|HTML_showTableEnd|HTML_showTableHeader|HTML_showTableHeading|HTML_showTableRow|HTML_showTitle|HTML_sourceViewer|HTML_storableCheckBox|HTML_storableInput|HTML_storableMultiSelection|HTML_storableSelection|HTML_submit|HTML_submitCheck|HTML_submitDefine|HTML_submitImg|HTML_textArea|HTML_uploadFile|HTML_urlButton|HTML_waitAnimation|HTML_weekdayTimeChooser|HWINFO_getHDSize|HWINFO_getMemory|HWINFO_printPartitions|I18N_addClientLanguageToCache|I18N_addLanguage|I18N_cacheClientLanguages|I18N_cacheWebinterfaceLanguages|I18N_convertToHumanReadableName|I18N_countCachedLanguages|I18N_getAllCachedLanguages|I18N_getAllTranslationsForAllVariables|I18N_getAllTranslationsForVariable|I18N_getHumanReadableDayHourMinute|I18N_getLangVars|I18N_getWeekDayArray|I18N_listClientLanguages|I18N_listWebinterfaceLanguages|I18N_m23instLanguage|I18N_number_format|IMG_clientCreate|IMG_clientRestore|IMG_clientRestoreAll|IMG_getAllImages|IMG_getAllImagesSel|IMG_getAllMBRs|IMG_getExtractedSize|IMG_getFormatCompressionFromFile|IMG_getImageFormatSelection|IMG_installMBR|IMG_makeExtension|IMG_recodeToNameOfMBR|IMG_ReconfigureEFI|IMG_serverCreate|IMG_setFilenameSize|IMG_showCreateImage|IMG_showImageManagement|IMG_storeMBR|KDE_install|KDE_installLoginManager|KDE_prepare|LDAP_addNewGroupID|LDAP_addNewID|LDAP_addNewUserID|LDAP_addPosix|LDAP_addServerTophpLdapAdmin|LDAP_checkphpLdapAdminConfiguration|LDAP_connectServer|LDAP_deleteID|LDAP_delServerFromphpLdapAdmin|LDAP_fqdn2dn|LDAP_getFreeGroupIDs|LDAP_getFreeIDs|LDAP_getFreeUserIDs|LDAP_getNextGroupID|LDAP_getNextID|LDAP_getNextUserID|LDAP_getTypes|LDAP_getValueFromConfigLine|LDAP_I18NLdapType|LDAP_installServer|LDAP_listServers|LDAP_loadServer|LDAP_makeConnection|LDAP_matchLDAPserver|LDAP_showServerManagementDialog|LVM_createLogicalVolume|LVM_createVolumeGroup|LVM_init|LXDE_install|m23SHARED_activate|m23SHARED_addExtraBill|m23SHARED_allUserDBQuery|m23SHARED_billDateSelection|m23SHARED_blockAccount|m23SHARED_calculateBill|m23SHARED_changeClientAmount|m23SHARED_changeEmailDialog|m23SHARED_changePasswordDialog|m23SHARED_DBname|m23SHARED_deleteBills|m23SHARED_deleteUserdataFromDB|m23SHARED_downloadBillPDFLink|m23SHARED_evaluationDaysLeft|m23SHARED_evaluationEndDate|m23SHARED_generateActivationKey|m23SHARED_generateCustomerNumber|m23SHARED_getActivationKey|m23SHARED_getAllm23sharedUsers|m23SHARED_getBillDates|m23SHARED_getCompleteClientName|m23SHARED_getCurrentUser|m23SHARED_getCustomerAddress|m23SHARED_getCustomerBankHTML|m23SHARED_getCustomerEmail|m23SHARED_getCustomerLanguage|m23SHARED_getCustomerNr|m23SHARED_getDBnameByClient|m23SHARED_getDBnameByInterface|m23SHARED_getInformationForBootingYourClientLink|m23SHARED_getLicenseType|m23SHARED_getMonthlyFee|m23SHARED_getPayTypeArray|m23SHARED_getPayTypeHumanReadable|m23SHARED_getRealName|m23SHARED_getSalutationHeadline|m23SHARED_getSalutationsArray|m23SHARED_getServerIP|m23SHARED_getUserByCustomerNr|m23SHARED_getUserByInfo|m23SHARED_getUserByResident|m23SHARED_init|m23SHARED_isAccountBlocked|m23SHARED_isMarkedForDeletion|m23SHARED_markBillMailAsSent|m23SHARED_markForDeletion|m23SHARED_new|m23SHARED_paidClients|m23SHARED_pdfBill|m23SHARED_prepareBillMailSending|m23SHARED_priceFormater|m23SHARED_sendActivationMail|m23SHARED_sendAdminMail|m23SHARED_sendAllBillMails|m23SHARED_sendWelcomeMail|m23SHARED_setAddress|m23SHARED_setBankAccount|m23SHARED_setCustomerEmail|m23SHARED_setCustomerLanguage|m23SHARED_setLicense|m23SHARED_setRealName|m23SHARED_showAddExtraBillDialog|m23SHARED_showAdminDialog|m23SHARED_showBill|m23SHARED_showBillDialog|m23SHARED_showBootMediaDownloadDialog|m23SHARED_showDonationDialog|m23SHARED_showLicenseDialog|m23SHARED_showPriceListTable|m23SHARED_switchUser|m23SHARED_unusedPaidClientsAvailable|MAIL_AESDecode|MAIL_AESencode|MAIL_attach|MAIL_cryptMailServer|MAIL_deleteGPGKey|MAIL_getGpgKeyList|MAIL_getHeader|MAIL_getKeyFromeMailAddress|MAIL_gpgCheckKey|MAIL_gpgGettKey|MAIL_gpgMail|MAIL_gpgSign|MAIL_gpgSignClear|MAIL_gpgSignDetached|MAIL_importGPGKey|MAIL_manageGPGKeysDialog|MAIL_sendAESMail|MAIL_sendMail|MASS_checkAndSaveFields|MASS_closeDBFile|MASS_EGKradioBoxes|MASS_FHradioBoxes|MASS_generateClientNames|MASS_generateNetmask|MASS_getAllFromFile|MASS_getLongestLength|MASS_getXProperties|MASS_I18NTokey|MASS_ipGenerator|MASS_keyToI18N|MASS_loginGenerator|MASS_minMaxIP|MASS_openDBFile|MASS_passGenerator|MASS_propertyKeys|MASS_readDBFile|MASS_readDBFileRaw|MASS_saveGeneratorOptions|MASS_showFileFormatDialog|MASS_showFileHandDialog|MASS_showGeneratorOptions|MASS_showOverview|MASS_showTableDefinition|MASS_startInstall|MENU_endGroup|MENU_showEntry|MENU_startGroup|MSESS_load|MSESS_save|MSG_DeActivateBlogDialog|MSG_getm23DevelopmentBlog|MSG_getm23UpdateFeed|MSG_getRSSFeed|MSG_placeOrReturnMessageBox|MSG_show|MSG_showAddJobsInfo|MSG_showEmergencyError|MSG_showError|MSG_showInfo|MSG_showMessageBox|MSG_showMessageBoxFooter|MSG_showMessageBoxHeader|MSG_showMessageBoxPlaceholder|MSG_showNewFeature|MSG_showUpdateInfo|MSG_showWarning|MSR_addMonitor|MSR_buildPoolFromClientDebs|MSR_buildPoolFromClientDebsCMD|MSR_clientChange|MSR_clientChangeCommand|MSR_clientSettings|MSR_clientSideBase64Encode|MSR_copyClientPackageStatus|MSR_CopyClientPackageStatusCommand|MSR_copyDiffFileFromClient|MSR_curDynIP|MSR_curDynIPCommand|MSR_decodeClientSideBase64|MSR_decodeMessage|MSR_genericSendCommand|MSR_genSendBinayFileCommand|MSR_genSendCommand|MSR_getClientSettingsCommand|MSR_getm23clientIDCMD|MSR_importDiffFile|MSR_importLog|MSR_importPackageStatus|MSR_importPartHwData|MSR_importStatusFile|MSR_listMonitors|MSR_logCommand|MSR_m23ImagerMBR|MSR_m23ImagerSize|MSR_markm23normalAsDone|MSR_partHwDataCommand|MSR_setOnline|MSR_statusBar|MSR_statusBarCommand|MSR_statusBarInc|MSR_statusBarIncCommand|MSR_statusFileCommand|PDF_init|PDF_output|PDF_showTableEnd|PDF_showTableHeader|PDF_showTableRow|PKG_|PKG_acceptJobs|PKG_addAPTConfigFiles|PKG_addDebianUser|PKG_addHSUser|PKG_addJob|PKG_addNormalJob|PKG_addNormalPackages|PKG_addNormalPackagesToWait4Aac|PKG_addPackageSelection|PKG_addPackageToPackageselection|PKG_addRecommendPackages|PKG_addRemovePackagesToWait4Aac|PKG_addShutdownOrRebootPackage|PKG_addShutdownPackage|PKG_addSpecialPackages|PKG_addSpecialPackagesToWait4Aac|PKG_addStatusJob|PKG_addUbuntuUser|PKG_addUser|PKG_addWait4AccJob|PKG_baseSysDownloadedCompletelyTom23Server|PKGBUILDER_listFiles|PKGBUILDER_showDialog|PKGBUILDER_showKeySelectionDialog|PKGBUILDER_showUploadDialog|PKGBUILDER_tar2deb|PKG_changeClientJobsStatus|PKG_changeClientPackageAction|PKG_changeInstallReasonSelectedPackages|PKG_changePrioritySelectedPackages|PKG_cleanPackageLine|PKG_closeSearch|PKG_combinem23normal|PKG_convertPackagesToRepository|PKG_copyPackagesToClient|PKG_copyWait4accPackagesToClient|PKG_countJobs|PKG_countJobsWithStatus|PKG_countPackages|PKG_countSelectedpackages|PKG_countSpecialPackages|PKG_countWaitingJobs|PKG_decodeDebconfDescription|PKG_deletePackageselection|PKG_deletePackagesList|PKG_discardJob|PKG_discardJobs|PKG_discardNormalJob|PKG_discardRemoveJob|PKG_downloadBaseSysTom23Server|PKG_downloadPool|PKG_executeOnClientJobs|PKG_exportSelectedPackages|PKG_fastGetInstalledPackages|PKG_fastGetNewInstalledPackages|PKG_genPackageSearchCacheFileCMD|PKG_getAllPackageSelections|PKG_getAllParams|PKG_getAptArchOptions|PKG_getClientbyPackageID|PKG_getClientIDbyPackageID|PKG_getClientjobsStatus|PKG_getClientPackages|PKG_getClientsByPackages|PKG_getClientsWithPackage|PKG_getClientsWithWaitingJobs|PKG_getDebootStrapBasePackages|PKG_getDebootstrapCacheFilename|PKG_getDebootstrapCacheServerFile|PKG_getDebootstrapCacheServerURL|PKG_getDebootstrapCacheSfURL|PKG_getDescription|PKG_getDuplicatedIdenticalJobs|PKG_getHigestIDOfSpecialPackage|PKG_getInfoFromPackageID|PKG_getKernels|PKG_getLongPackageDescription|PKG_getNextPackage|PKG_getPackageDescriptionSize|PKG_getPackageID|PKG_getPackageIDsByName|PKG_getPackageName|PKG_getPackageParams|PKG_getPackageParamsVar|PKG_getPackagesListMarker|PKG_getPackageStatusCSV|PKG_getRecommendPackageAllInstalledSize|PKG_getSearchCacheFileName|PKG_getSpecialPackageDescription|PKG_getSpecialPackageInfo|PKG_getSpecialPackagePriority|PKG_hasOptions|PKG_importSelectedPackagesFromFile|PKG_isReconfiguredWithExtraDistr|PKG_isSpecialPackageAvailableForClient|PKG_listPackages|PKG_listParams|PKG_listRecommendPackages|PKG_listRecommendSubPackages|PKG_listSelectedpackages|PKG_listSpecialpackages|PKG_loadPackagesList|PKG_multiPackageSelectionsSelection|PKG_ncTarDebsFromClientToServer_Client|PKG_ncTarDebsFromClientToServer_Server|PKG_OptionPageGetValue|PKG_OptionPageHeader|PKG_OptionPageHeader2|PKG_OptionPageRender|PKG_OptionPageRender2|PKG_OptionPageSaveAlsParameters|PKG_OptionPageTail|PKG_OptionPageTail2|PKG_pkgUpdateCacheOnServer|PKG_preparePackageDir|PKG_preparePool|PKG_previewInstallationDeinstallation|PKG_previewInstallDeinstall|PKG_previewUpdateSystem|PKG_printStatus|PKG_recountAllClientPackages|PKG_remNormalPackages|PKG_removeDuplicatedIdenticalJobs|PKG_removeFromJobList|PKG_removeSpecialFromJobList|PKG_rmAllSpecialPackagesByName|PKG_rmNormalJob|PKG_rmSelectedPackages|PKG_rsyncDebsFromClientToServerRaus|PKG_savePackageselection|PKG_savePackagesList|PKG_searchFor|PKG_searchPackageSearchCacheFileCMD|PKG_setAllParams|PKG_setClientPackageInstalledOK|PKG_setClientPackageWait4Rm|PKG_showAllPackageSelections|PKG_showPreviewInstallationDeinstallation|PKG_showPreviewUpdateSystem|PKG_translateClientjobsStatus|PKG_translateClientPackageStatus|PKG_updatePackageInfo|PKG_updatePackageSearchCacheFile|PKG_updateSourcesListAtAllClients|PLG_checkOverwriting|PLG_DBInstall|PLG_delete|PLG_getFilename|PLG_getPLGAuthor|PLG_getPLGClientRequires|PLG_getPLGName|PLG_getPLGPage|PLG_getPLGUpdateURL|PLG_getPLGVersion|PLG_getPlugin|PLG_getPluginValue|PLG_getTempDir|PLG_getUpdateFile|PLG_install|PLG_isPluginSelected|PLG_listInfofile|PLG_listMenuPlugins|PLG_realInstall|PLG_realUpdate|PLG_showDownloadStatus|PLG_showPluginOverview|PLG_update|POOL_create|POOL_createExtendedPackageIndex|POOL_delete|POOL_download|POOL_getCDDistributionRelease|POOL_getDir|POOL_getPools|POOL_getProperty|POOL_getSize|POOL_makeRepository|POOL_prepare|POOL_readCD|POOL_selectPoolType|POOL_setProperty|POOL_showCreatePackageIndex|POOL_showDownloadStatus|POOL_showLoadDeleteCreate|POOL_showReadCD|POOL_showSourcesList|PREF_delete|PREF_exists|PREF_getAllValues|PREF_getClientPreferences|PREF_getValue|PREF_loadAllPreferenceValues|PREF_preferenceLoadManagerHandler|PREF_preferenceSaveManagerHandler|PREF_putAllOptions|PREF_putValue|PREF_saveAllPreferenceValues|PREF_showPreferenceManager|print_r2|private_and|private_lshift|private_rshift|RAID_create|RAID_createMdadmConf|RAS_chatWindow|RAS_getChatUser|RAS_gethttpPassword|RAS_getMsg|RAS_getRandomKey|RAS_getTunnelPid|RAS_importMsg|RAS_isInstalled|RAS_isRunning|RAS_remove|RAS_showAccountDataImport|RAS_showRegistration|RAS_showStartStopRemove|RAS_start|RAS_tunnelOpenHTMLStatus|RAS_writeTunnelScripts|readbytes|RMV_exists4IP|RMV_get|RMV_get4IP|RMV_rm|RMV_rm4IP|RMV_rm_old|RMV_set|RMV_set4IP|SCREDIT_correctScriptFilename|SCREDIT_newScriptTemplate|SCREDIT_showEditor|SERVER_addAdmin|SERVER_addEtcHosts|SERVER_addLineToFile|SERVER_addToHtpasswd|SERVER_apacheInfo|SERVERBACKUP_backupOverviewDialog|SERVERBACKUP_getBackupConfiguration|SERVERBACKUP_getBackupList|SERVERBACKUP_rmBackup|SERVERBACKUP_runBackupNowDialog|SERVERBACKUP_showConfigurationDialog|SERVERBACKUP_storeBackupConfiguration|SERVER_changeHtpasswd|SERVER_checkDiskFree|SERVER_checkDownload|SERVER_checkKernel|SERVER_checkPackageInstalled|SERVER_checkRunInVM|SERVER_commandAvailable|SERVER_daemonStartStop|SERVER_delAdmin|SERVER_delEtcHosts|SERVER_deleteFile|SERVER_delFromHtpasswd|SERVER_delLineFromFile|SERVER_dhcpInfo|SERVER_dhcpServerInNetWarn|SERVER_dynamicIPWarn|SERVER_existsServerSetting|SERVER_fileExists|SERVER_getAptGetInstallCommand|SERVER_getFileContents|SERVER_getInstallationMedium|SERVER_getOS|SERVER_getPublicSSHKeyOfm23Server|SERVER_getServerBoolSetting|SERVER_getServerSetting|SERVER_importGPGPackageSignKey|SERVER_insertLineNumber|SERVER_installTool|SERVER_installToolInBackground|SERVER_isClientOnlineStatusEnabled|SERVER_isInstallReasonEnabled|SERVER_isLiveLogDisabled|SERVER_isProgramRunning|SERVER_isSSLCertCheckDisabled|SERVER_isUpdatePackageInfosDisabled|SERVER_killBackgroundJob|SERVER_killPID|SERVER_LDAPInfo|SERVER_logLocalScreenSessionToFile|SERVER_multiMkDir|SERVER_mysqlInfo|SERVER_programmStatus|SERVER_programmStatusTableHeader|SERVER_putFileContents|SERVER_rootFreeSpace|SERVER_runInBackground|SERVER_runningInBackground|SERVER_runningInScreen|SERVER_sendScriptToSF|SERVER_setClientOnlineStatusEnabled|SERVER_setInstallReasonEnabled|SERVER_setLiveLogDisabled|SERVER_setServerBoolSetting|SERVER_setServerSetting|SERVER_setSSLCertCheckDisabled|SERVER_setUpdatePackageInfosDisabled|SERVER_startUpdateClientOnlineInDBBackgroundJob|SERVER_tmpNotWritable|SRCLST_alternativeFS|SRCLST_checkList|SRCLST_checkm23debsMirror|SRCLST_cleanList|SRCLST_clientUsesEfiButSourcesListDoesntSupportEfi|SRCLST_delete|SRCLST_doesDistrSupportEFI|SRCLST_genList|SRCLST_genSelection|SRCLST_getAddToFile|SRCLST_getArchitectures|SRCLST_getDescription|SRCLST_getDesktopList|SRCLST_getExportedListNames|SRCLST_getListnames|SRCLST_getListnamesWithEfiSupport|SRCLST_getMirror|SRCLST_getParameter|SRCLST_getRelease|SRCLST_getStorageFS|SRCLST_getValue|SRCLST_getWorkingm23debsMirror|SRCLST_isArchAvailable|SRCLST_loadSourceList|SRCLST_loadSourceListFromDB|SRCLST_matchList|SRCLST_packageInformationChangeInformationHumanReadable|SRCLST_packageInformationChangeTime|SRCLST_packageInformationOlderThan|SRCLST_possiblem23debsMirrors|SRCLST_querySourceslists|SRCLST_saveArchitectures|SRCLST_saveList|SRCLST_showAlternativeArchitectureSelection|SRCLST_showDesktopsSel|SRCLST_showEditor|SRCLST_showErrorIfClientUsesEfiButSourcesListDoesntSupportEfi|SRCLST_sourceListExists|SRCLST_supportedFS|TRINITY_install|TRINITY_installLoginManager|UBUNTU_desktopInstall|UBUNTU_fixAfterBaseInstall|UBUNTU_fixBeforeBaseInstall|UBUNTU_installLanguagePacks|UCS_addClient|UCS_addLDAPUser|UCS_addUCSClientTom23ClientPreferences|UCS_createNetworkObject|UCS_delClient|UCS_delUCSClientFromm23ClientPreferences|UCS_disableClientPXEBoot|UCS_enableClientLDAP|UCS_enableClientPXEBoot|UCS_ensureNetworkObjectExists|UCS_getAllClientNamesLDAP|UCS_getClientLDAPInfo|UCS_getEtc_ucr_master|UCS_getFirstElementFromDN|UCS_getGenericNetworkName|UCS_getNetworkLDAPInfo|UCS_getPrefnameByClient|UCS_getUDMInfo|UCS_getUsedIPs|UCS_modifyClientIP|UCS_networkObjectExists|UCS_openFirewallPort|UCS_setClientDistrAndRelease|UCS_udmSuccessOrErrorMessage|UPDATE_doUpdate|UPDATE_getInfo|UPDATE_getUrl|UPDATE_running|VM_activateNetboot|VM_activateNetbootCMD|VM_captureVMWindowAsMovie|VM_CloudStack_available|VM_CloudStackCheckConstants|VM_CloudStackClientName2ClientID|VM_CloudStackConfigGUI|VM_CloudStackCreateVM|VM_CloudStackDeleteClientVM|VM_CloudStackDisablePortForwarding|VM_CloudStackEnablePortForwarding|VM_CloudStack_getObject|VM_CloudStack_getServerIP|VM_CloudStack_getVersion|VM_CloudStackGetVMStatus|VM_CloudStackNetBootActivate|VM_CloudStackSendSetVisualURL|VM_CloudStackStartVM|VM_CloudStackStopVM|VM_CloudStackUploadIso|VM_CloudStackWriteConfFile|VM_convertSwitchStatusInfo|VM_createDiskImage|VM_createVM|VM_delete|VM_delVMCMD|VM_downloadedVBoxAddons|VM_downloadVBOXaddons|VM_generateVBOXaddonDownloadCMD|VM_getAllVMHosts|VM_getHTMLStatusBlock|VM_getStatus|VM_getSWandHost|VM_getVBoxAddonDefaultVersion|VM_getVBoxVersion|VM_GUIstepCheckHost|VM_GUIstepCreateCloudStackVM|VM_GUIstepCreateGuest|VM_GUIstepSelectHost|VM_insertBootISO|VM_isCloudStackClient|VM_isHost|VM_listDownloadableVBoxAddons|VM_parseStatus|VM_parseVBOXdisk|VM_parseVBOXNic|VM_parseVBOXstate|VM_pauseVM|VM_rebootAndActivateNetboot|VM_rebootAndDisableNetbootAfterInstall|VM_rebootChangeBootDevice|VM_restoreSnapshot|VM_resumeVM|VM_setGuestInDB|VM_setHostInDB|VM_setVBoxAddonAsDefault|VM_setVisualURL|VM_shutdownAndDisableNetboot|VM_shutdownAndDisableNetbootAfterInstall|VM_startVM|VM_startVMCommandFile|VM_startVMInExistingXSession|VM_status|VM_statusIcons|VM_stopVM|VM_stopVMCommandFile|VM_VBOXaddonDownloadDialog|VM_vmSwNr2Name|VM_wasVBoxAddonDownloaded|VM_webAction|xhprof_init|xhprof_shutdown)\b/g, output : '<u>$1</u>' }, // m23
	{ input : /([^:])\/\/(.*?)(<br|<\/P)/g, output : '$1<i>//$2</i>$3' }, // php comments //
	{ input : /([^:])#(.*?)(<br|<\/P)/g, output : '$1<i>#$2</i>$3' }, // php comments #
	{ input : /\/\*(.*?)\*\//g, output : '<i>/*$1*/</i>' }, // php comments /* */
	{ input : /(&lt;!--.*?--&gt.)/g, output : '<big>$1</big>' } // html comments
]

Language.snippets = [
	{ input : 'if', output : 'if($0){\n\t\n}' },
	{ input : 'ifelse', output : 'if($0){\n\t\n}\nelse{\n\t\n}' },
	{ input : 'else', output : '}\nelse {\n\t' },
	{ input : 'elseif', output : '}\nelseif($0) {\n\t' },
	{ input : 'do', output : 'do{\n\t$0\n}\nwhile();' },
	{ input : 'inc', output : 'include_once("$0");' },
	{ input : 'fun', output : 'function $0(){\n\t\n}' },	
	{ input : 'func', output : 'function $0(){\n\t\n}' },	
	{ input : 'while', output : 'while($0){\n\t\n}' },
	{ input : 'for', output : 'for($0,,){\n\t\n}' },
	{ input : 'fore', output : 'foreach($0 as ){\n\t\n}' },
	{ input : 'foreach', output : 'foreach($0 as ){\n\t\n}' },
	{ input : 'echo', output : 'echo \'$0\';' },
	{ input : 'switch', output : 'switch($0) {\n\tcase "": break;\n\tdefault: ;\n}' },
	{ input : 'case', output : 'case "$0" : break;' },
	{ input : 'ret0', output : 'return false;' },
	{ input : 'retf', output : 'return false;' },
	{ input : 'ret1', output : 'return true;' },
	{ input : 'rett', output : 'return true;' },
	{ input : 'ret', output : 'return $0;' },
	{ input : 'def', output : 'define(\'$0\',\'\');' },
	{ input : '<?', output : 'php\n$0\n?>' }
]

Language.complete = [
	{ input : '\'', output : '\'$0\'' },
	{ input : '"', output : '"$0"' },
	{ input : '(', output : '\($0\)' },
	{ input : '[', output : '\[$0\]' },
	{ input : '{', output : '{\n\t$0\n}' }		
]

Language.shortcuts = [
	{ input : '[space]', output : '&nbsp;' },
	{ input : '[enter]', output : '<br />' } ,
	{ input : '[j]', output : 'testing' },
	{ input : '[7]', output : '&amp;' }
]
